
(****************************************************************)
(*		Partitions an integer using                     *)
(*    a stack. Ref H. Alder-American Mathematical Monthly       *)
(*        by AA  8/24/90                                        *)
(****************************************************************)


PROGRAM Partition (Input, Output);

CONST StackSize = 25;

TYPE  StackPointer=^StackType;
      StackItem = Integer;
      StackType = Record
                Entry: StackItem;
                Link: StackPointer;
      End;

VAR   Stack:  StackPointer;
      Top,                 (* Pointer to top of stack *)
      StackSum,            (* Total of items on stack *)
      Number,              (* The number being partitioned *)
      Count,               (* the number of partitions *)
	 Diff:   Integer;     (* Auxilliary for filling stack *)
      Finished: Boolean;   (* Finished if nil *)

(***********************************************************)
(*              Function Empty                             *)
(*  PreCondition:  Assumes that the stack has been created *)
(***********************************************************)

FUNCTION Empty(VAR S: StackPointer): Boolean;
  BEGIN
      IF (Top = 0) THEN
         Empty:= True
      ELSE
         Empty:= False;
      END;

(************************************************************)
(*               Function Full                              *)
(*  PreCondition:  Assumes that the stack has been created  *)
(************************************************************)


FUNCTION Full (VAR S: StackType): Boolean;
      BEGIN
      IF (TOP = StackSize) THEN
         Full:= True
      ELSE
         Full:= False;
      END;


(****************************************************************)
(*      Procedure Initiation                                    *)
(*      The Stack is is to be initialized as empty              *)
(****************************************************************)

PROCEDURE Initiation (VAR S: StackPointer);
          BEGIN
            S:=Nil
          End;

(****************************************************************)
(*	Procedure WriteStack prints the entries on the	         *)
(*	stack from the bottom up. This yields a 	              *)
(*	partition in descending order.			 	         *)
(* PreCondition: Assumes thatr the stack has been created       *)
(****************************************************************)

PROCEDURE WriteStack (S:StackPointer);
          VAR  Index: StackPointer;
          BEGIN
               Index:=S;
               If (Index <> Nil) Then
               BEGIN
                    WriteStack (Index^.Link);
                    Write (Index^.Entry:1,' ');
                END
               End;
(****************************************************************)
(*                   Procedure PUSH                             *)
(*  Precondition: Stack Exists                                  *)
(*  Postcondition: If stack was not full, "Entry" has been put  *)
(*                 on top of stack.                             *)
(****************************************************************)

PROCEDURE Push (VAR S: StackPointer; Item: StackItem);
          VAR P: StackPointer;
          BEGIN
          IF (Top < StackSize) THEN
          BEGIN
             New (P);
             P^.Entry:= Item;
             P^.Link:= S;
             S:= P;
             Top:= Top + 1
           END
          ELSE Writeln ('Error: Stack Overflow');
          END;
      (****************************************************************)
      (*             Procedure POP                                    *)
      (*  PreCondition: Stack Exists                                  *)
      (*  Postcondition: If stack was not empty, the item on the top  *)
      (*                 of the stack has been removed and returned   *)
      (*                 as "Entry".                                  *)
      (****************************************************************)
PROCEDURE Pop(VAR S: StackPointer;VAR Entry:StackItem);
       VAR P:StackPointer;
          BEGIN
          IF (S=nil) THEN
          Writeln ('Attempt to Pop and empty stack') ELSE
          BEGIN
            P:=S;
            Entry:=S^.Entry;
            S:=S^.Link;
            Dispose (P);
            Top:= Top - 1;
          END
        END;
BEGIN		(*************** Partition ***************)
               (* Initialize stack and solicit number *)
      Initiation (Stack);
      Top:= 0;  (* Empty stack condition *)
      REPEAT
      Writeln ('Enter the integer to be partitioned, 1-',StackSize:1);
      Readln (Number);
      UNTIL Number IN [1..StackSize];
      Push (Stack,Number);
      StackSum:= Number;
      Writeln;
      WriteStack (Stack);            (* Write initial entry *)
      Count:= 1;
      IF (Number = 1) THEN
      Pop(Stack, Number);           (* Deal with special case *)
                                    (* to simplify the loop entry condition *)
      WHILE (not Empty(Stack)) DO
      BEGIN
           Stack^.Entry:= Stack^.Entry - 1;	(* Decrement top entry to begin forming *)
           StackSum:= StackSum - 1;		(* new partition  *)
      REPEAT
           Diff:= Number  - StackSum;	(* Fill stack with entries *)
           IF (Stack^.Entry >= Diff)   THEN	(* no larger than Stacktop *)
           Push(Stack,Diff)
          ELSE
           Push(Stack,Stack^.Entry);
           StackSum:= StackSum + Stack^.Entry;
          UNTIL  StackSum = Number;
                 WriteStack (Stack);            (* Display complete partition  *)
                 Writeln;
                 Count:= Count  + 1;            (* and count it. *)
                 Finished:=False;
          WHILE  (Not Empty(Stack) AND Not Finished) DO
      BEGIN
          IF (Stack^.Entry=1) THEN
          BEGIN
           Pop(Stack,Diff);		(* Discard 1's from stack *)
           StackSum:= StackSum - 1;
          END
          ELSE
           Finished:=True
          END
      END;	(* Main WHILE Loop *)

          Writeln ;
          Writeln(' THERE ARE ',Count:0,' PARTITIONS OF ',Number:0);
          Writeln;
          END.
    		(******************  Partition *****************)
